/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

/* This is a trivial JNI example where we use a native method
 * to return a new VM String. See the corresponding Java source
 * file located at:
 *
 *   apps/samples/hello-jni/project/src/com/example/hellojni/HelloJni.java
 */


#include <jni.h>
#include <stdio.h>
#include <gst/gst.h>
#include <android/log.h>


GST_PLUGIN_STATIC_DECLARE(coreelements);
GST_PLUGIN_STATIC_DECLARE(androidmedia);

jstring
Java_com_example_hellojni_HelloJni_stringFromJNI( JNIEnv* env,
                                                  jobject thiz )
{

    const gchar *nano_str;
    guint major, minor, micro, nano;
    char str[1024] = "default";
    GError *pError = NULL;
    GstPipeline *pPipe = NULL;
    GstElementFactory *factory;
    GstElement *pElement;
    GList* plugins; /* The head of the plug-in list */
    GList* pnode;   /* The currently viewed node */

    gst_init (NULL, NULL);
    gst_version (&major, &minor, &micro, &nano);

    if (nano == 1)
        nano_str = "(CVS)";
    else if (nano == 2)
        nano_str = "(Prerelease)";
    else
        nano_str = "";

    GST_PLUGIN_STATIC_REGISTER(coreelements);
    GST_PLUGIN_STATIC_REGISTER(androidmedia);

    pPipe = gst_parse_launch("ahcsrc device=1 ! fakesink", &pError);

    factory = gst_element_factory_find("ahcsrc");
    if(factory)
        pElement = gst_element_factory_create (factory, "source");
        //pElement = gst_element_factory_make ("ahcsrc", "source");

    sprintf (str, "This program is linked against GStreamer %d.%d.%d %s\n"
             "pipeline %x\n"
             "factory %x\n"
             "element(ahcsrc) %x\n"
             "Error: %s\n",
             major, minor, micro, nano_str,
             (void*) pPipe, (void*) factory, (void*) pElement, pError ? pError->message : "None");

    plugins = pnode = gst_registry_get_plugin_list(gst_registry_get());
    while(pnode)
    {
        GstPlugin* plugin = (GstPlugin*)pnode->data;
        sprintf (str, "%s%s\n", str, gst_plugin_get_name(plugin));
        pnode = g_list_next(pnode);
    }

    __android_log_write(ANDROID_LOG_INFO, "gst-test", str);//Or ANDROID_LOG_INFO, ...
    return (*env)->NewStringUTF(env, str);

    // sprintf(sVer, "Cairo version: %d", ver);

    /*if (ver > 0)
        return (*env)->NewStringUTF(env, "libCairo ver > 0");
    else
        return (*env)->NewStringUTF(env, "libCairo ver < 0");*/

    // return (*env)->NewStringUTF(env, sVer);
    // return (*env)->NewStringUTF(env, "Hello from JNI !  Compiled with ABI " ABI ".");
}

//#if defined(__arm__)
//  #if defined(__ARM_ARCH_7A__)
//    #if defined(__ARM_NEON__)
//      #if defined(__ARM_PCS_VFP)
//        #define ABI "armeabi-v7a/NEON (hard-float)"
//      #else
//        #define ABI "armeabi-v7a/NEON"
//      #endif
//    #else
//      #if defined(__ARM_PCS_VFP)
//        #define ABI "armeabi-v7a (hard-float)"
//      #else
//        #define ABI "armeabi-v7a"
//      #endif
//    #endif
//  #else
//   #define ABI "armeabi"
//  #endif
//#elif defined(__i386__)
//   #define ABI "x86"
//#elif defined(__x86_64__)
//   #define ABI "x86_64"
//#elif defined(__mips64)  /* mips64el-* toolchain defines __mips__ too */
//   #define ABI "mips64"
//#elif defined(__mips__)
//   #define ABI "mips"
//#elif defined(__aarch64__)
//   #define ABI "arm64-v8a"
//#error "wtf arm64"
//#else
//   #define ABI "unknown"
//#endif